const Deliveries = require("../Models/Deliveries");

const getDeliveries = async (req, res) => {
  try {
    const deliveries = await Deliveries.findAll({
      order: [["id", "DESC"]],
    });

    return res.status(200).json({ deliveries });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const completeDelivery = async (req, res) => {
  const { id } = req.body;
  if (!id) return res.status(400).json({ mssg: "Invalid ID" });
  try {
    const delivery = await Deliveries.findByPk(id);
    if (!delivery) return res.status(400).json({ mssg: "Invalid ID" });
    if (delivery.dataValues.isCompleted)
      return res.status(400).json({
        mssg: "The delivery is already completed",
      });

    const deliveries = await Deliveries.update(
      {
        isCompleted: true,
      },
      {
        where: {
          id,
        },
      }
    );

    return res.status(200).json({ mssg: "Delivery has been completed" });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const addDelivery = async (req, res) => {
  const { name, date, orderName } = req.body;
  if (!name) return res.status(400).json({ mssg: "Name field is required" });
  if (!date) return res.status(400).json({ mssg: "Date field is required" });
  if (!orderName)
    return res.status(400).json({ mssg: "Order name field is required" });

  try {
    const delivery = await Deliveries.create({
      name,
      date,
      orderName,
    });

    return res.status(200).json({ delivery });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const editDelivery = async (req, res) => {
  const { name, date, id } = req.body;
  if (!id) return res.status(400).json({ mssg: "Invalid ID" });
  if (!name) return res.status(400).json({ mssg: "Name field is required" });
  if (!date) return res.status(400).json({ mssg: "Date field is required" });

  try {
    const delivery = await Deliveries.findByPk(id);
    if (!delivery) return res.status(400).json({ mssg: "Invalid ID" });

    await Deliveries.update(
      {
        name,
        date,
      },
      {
        where: {
          id,
        },
      }
    );

    return res.status(200).json({
      mssg: "Delivery details has been edited successfully",
    });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const deleteDeliveries = async (req, res) => {
  const { id } = req.body;
  if (!id) return res.status(400).json({ mssg: "Invalid ID" });
  try {
    const delivery = await Deliveries.findByPk(id);
    if (!delivery) return res.status(400).json({ mssg: "Invalid ID" });

    await Deliveries.destroy({
      where: {
        id,
      },
    });

    return res.status(200).json({ mssg: "Delivery has been deleted" });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

module.exports = {
  getDeliveries,
  addDelivery,
  editDelivery,
  deleteDeliveries,
  completeDelivery,
};
