const Orders = require("../Models/Orders");
const SysManager = require("../Models/SysManager");

const getOrders = async (req, res) => {
  try {
    const orders = await Orders.findAll({
      order: [["id", "DESC"]],
    });

    return res.status(200).json({ orders });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const getPendingOrders = async (req, res) => {
  try {
    const orders = await Orders.findAll({
      order: [["id", "DESC"]],
      where: {
        isCompleted: false,
      },
    });

    return res.status(200).json({ orders });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const getCompletedOrders = async (req, res) => {
  try {
    const orders = await Orders.findAll({
      order: [["id", "DESC"]],
      where: {
        isCompleted: true,
      },
    });

    return res.status(200).json({ orders });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const completeOrder = async (req, res) => {
  const { id, tax } = req.body;
  if (!id) return res.status(400).json({ mssg: "Invalid ID" });
  if (!tax) return res.status(400).json({ mssg: "Please fill in the tax" });

  try {
    const order = await Orders.findByPk(id);
    if (!order) return res.status(400).json({ mssg: "Invalid ID" });
    if (order.dataValues.isCompleted)
      return res.status(400).json({ mssg: "The order is already completed" });

    const sysManager = await SysManager.findByPk(1);
    await SysManager.update(
      {
        capital:
          sysManager.dataValues.capital * 1 + order.dataValues.sellerPrice * 1,
        profit:
          sysManager.dataValues.profit * 1 +
          (order.dataValues.clientPrice * 1 -
            order.dataValues.sellerPrice * 1 -
            tax),
      },
      { where: { id: 1 } }
    );

    await Orders.update(
      {
        isCompleted: true,
        tax,
      },
      {
        where: {
          id,
        },
      }
    );

    return res.status(200).json({ mssg: "Order has been completed" });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const addOrder = async (req, res) => {
  const { name, link, qty, sellerPrice, clientPrice, clientName } = req.body;
  if (!name) return res.status(400).json({ mssg: "Name field is required" });
  if (!link) return res.status(400).json({ mssg: "Link field is required" });
  if (!qty) return res.status(400).json({ mssg: "Quantity field is required" });
  if (!sellerPrice)
    return res.status(400).json({ mssg: "Saler price field is required" });
  if (!clientPrice)
    return res.status(400).json({ mssg: "Client price field is required" });
  if (!clientName)
    return res.status(400).json({ mssg: "Client name field is required" });

  try {
    const order = await Orders.create({
      name,
      link,
      qty,
      sellerPrice,
      clientPrice,
      clientName,
    });

    return res.status(200).json({ order });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const editOrder = async (req, res) => {
  const { name, link, qty, sellerPrice, clientPrice, id } = req.body;
  if (!id) return res.status(400).json({ mssg: "Invalid ID" });
  if (!name) return res.status(400).json({ mssg: "Name field is required" });
  if (!link) return res.status(400).json({ mssg: "Link field is required" });
  if (!qty) return res.status(400).json({ mssg: "Quantity field is required" });
  if (!sellerPrice)
    return res.status(400).json({ mssg: "Saler price field is required" });
  if (!clientPrice)
    return res.status(400).json({ mssg: "Client price field is required" });

  try {
    const order = await Orders.findByPk(id);
    if (!order) return res.status(400).json({ mssg: "Invalid ID" });

    await Orders.update(
      {
        name,
        link,
        qty,
        sellerPrice,
        clientPrice,
      },
      {
        where: {
          id,
        },
      }
    );

    return res.status(200).json({
      mssg: "Order details has been edited successfully",
    });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

const deleteOrders = async (req, res) => {
  const { id } = req.body;
  if (!id) return res.status(400).json({ mssg: "Invalid ID" });

  try {
    const order = await Orders.findByPk(id);
    if (!order) return res.status(400).json({ mssg: "Invalid ID" });

    await Orders.destroy({
      where: {
        id,
      },
    });

    return res.status(200).json({ mssg: "Order has been deleted" });
  } catch (e) {
    return res.status(400).json({ mssg: e.message });
  }
};

module.exports = {
  getOrders,
  addOrder,
  editOrder,
  deleteOrders,
  getPendingOrders,
  getCompletedOrders,
  completeOrder,
};
